#!/usr/bin/python

import httplib
import socket
import ssl


class HTTPSSignedConnection(httplib.HTTPSConnection):
    """ Class to make a HTTPS connection, signed by a valid/trusted CA root certificate, 
        with support for full client-based SSL/TLS Authentication"""


    def __init__(self, host, port=None, key_file=None, cert_file=None, ca_file=None):
        httplib.HTTPSConnection.__init__(self, host, key_file=key_file, cert_file=cert_file)
        self.ca_file = ca_file


    def connect(self):
        """ Connect to a host on a given (SSL) port.
            If ca_file is pointing somewhere, use it to check Server Certificate.

            Redefined/copied and extended from httplib.py:1105 (Python 2.6.x).
            This is needed to pass cert_reqs=ssl.CERT_REQUIRED as parameter to ssl.wrap_socket(),
            which forces SSL to check server certificate against our client certificate.
        """

        if hasattr(self, 'source_address'):
            sock = socket.create_connection((self.host, self.port), self.timeout, self.source_address)
        else:
            sock = socket.create_connection((self.host, self.port), self.timeout)

        if self._tunnel_host:
            self.sock = sock
            self._tunnel()

        # If there's no CA File, don't force Server Certificate Check
        if hasattr(self,'ca_file'):
            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file, ca_certs=self.ca_file, 
                                           cert_reqs=ssl.CERT_REQUIRED, ssl_version=ssl.PROTOCOL_TLSv1,
                                           do_handshake_on_connect=False)
        else:
            # Fallback to unsigned HTTPSConnection
            self.sock = ssl.wrap_socket(sock, self.key_file, self.cert_file, ca_certs=None,
                                           cert_reqs=ssl.CERT_NONE, ssl_version=ssl.PROTOCOL_TLSv1,
                                           do_handshake_on_connect=False)




if __name__ == '__main__':
    # Little test-case of our class
    import sys
    if len(sys.argv) != 7:
        print 'usage: python %s <host> <service_url> <cacert_file> <cert_file> <key_file> <srv_file>' % sys.argv[0]
        sys.exit(1)
    else:
        server, service_url, cacertfilename, certfilename, keyfilename, srvfilename = sys.argv[1:]


    # Start from here !!!

    cafile   = file(cacertfilename)
    soapfile = file(srvfilename)
    certfile = file(certfilename)
    keyfile  = file(keyfilename)

    try:
        ''' Versao 4.34.x '''
        con = HTTPSSignedConnection(server, key_file=keyfile.name, cert_file=certfile.name, ca_file=cafile.name)

        ''' Versao 4.33.x '''
        #con = httplib.HTTPSConnection(server, key_file=keyfile.name, cert_file=certfile.name)

    except Exception as error:
        print 'Error: cannot connect to server: %s => %s' % (server,error)

    request_url    = u'/'+service_url
    request_body   = soapfile.readlines()[0].encode()
    request_header = {'Content-Type': 'application/soap+xml;charset=UTF-8'}

    try:
        con.request('POST', request_url, body=request_body, headers=request_header)
    except Exception as error:
        print 'Error: Unknown error from server: %s => %s' % (server,error)
    finally:
        resp = con.getresponse()
	print resp.status
        if resp.status == 200:
            print 'Evebody is OK. Result: \n\n%s' % resp.read().decode('utf-8')
